
#include "english_6x8_pixel.h"
#include "font14x24.h"
//#include "write_chinese_string_pixel.h"

//
#define BIT(i)  (1<<i)  

#define SCLK_set   PORTD|=BIT(0)      //serial clock input
#define SCLK_clr   PORTD&=~BIT(0)  

#define SDIN_set  PORTD|=BIT(1)      //serial data input   
#define SDIN_clr  PORTD&=~BIT(1) 

#define LCD_DC_set   PORTD|=BIT(2)    //data/commande
#define LCD_DC_clr   PORTD&=~BIT(2)  

#define LCD_RST_set  PORTD|=BIT(3)     //external reset input
#define LCD_RST_clr  PORTD&=~BIT(3) 

uint8_t CurX,CurY,Inversion;


void delay_1us(void)                 //delay 1us
  {
     _delay_us(1);
  }

void delay_1ms(void)                 //delay 1ms
  {
    _delay_ms(1);
  }
  
void delay_nms(unsigned int n)       //delay nms
  {
	do { _delay_ms(1); }
	while(--n);
  }


/*-----------------------------------------------------------------------
LCD_write_byte    : write data to LCD

input parmenterdata    data
          command command

created date         2013-4-25
-----------------------------------------------------------------------*/
void LCD_write_byte(unsigned char dat, unsigned char command)
  {
    unsigned char i;
    
    if (command == 0)
       LCD_DC_clr;
    else
       LCD_DC_set;

		for(i=0;i<8;i++)
		{
			if(dat&0x80)
		     SDIN_set;
			else
		      SDIN_clr;
		    SCLK_clr;
			dat = dat << 1;
          	SCLK_set;
		}
  }

void LCD_write_data(unsigned char dat) {
  if (Inversion) dat = ~dat;
  LCD_write_byte(dat,1);
}

void LCD_clear(void)          // clear the LCD
  {
    unsigned int i;

    LCD_write_byte(0x0c, 0);			
    LCD_write_byte(0x80, 0);			

    for (i=0; i<504; i++)
	  {
       LCD_write_byte(0, 1);
	  }
    CurX=0;
    CurY=0;		
  }


void LCD_init(void)                 //LCD initialization
  {
//    DDRA|=0X1F;    

     LCD_RST_clr;
    delay_1us();
     LCD_RST_set;
    

    delay_1us();
  
    delay_1us();

    LCD_write_byte(0x21, 0);	// set LCD mode
    LCD_write_byte(0xb4, 0);	// set bias voltage
    LCD_write_byte(0x06, 0);	// temperature correction
    LCD_write_byte(0x14, 0);	// 1:48
    LCD_write_byte(0x20, 0);	// use bias command
    LCD_clear();	           // clear the LCD
    LCD_write_byte(0x0c, 0);	// set LCD mode,display normally
        
  }


/*-----------------------------------------------------------------------
LCD_set_XY              : Set the LCD coordinate functions

input parameterX       083
               Y        05

created date           2013-4-25
-----------------------------------------------------------------------*/
void LCD_set_XY(unsigned char X, unsigned char Y)
  {
    LCD_write_byte(0x40 | Y, 0);		// column
    LCD_write_byte(0x80 | X, 0);          	// row
	CurX=X;
	CurY=Y;
  }

/*-----------------------------------------------------------------------
LCD_write_char    : Display English characters

input parameterc   :char to display

created date         2013-4-25
-----------------------------------------------------------------------*/
void LCD_write_char(unsigned char c)
  {
    unsigned char line;

//    c -= 32;
    if ( (c >= 0x20) && (c <= 0x7F) )  //      ASCII[0x20-0x7F]
    {c -= 32;}
    else if ( c >= 0xC0 )   //      CP1251[0xC0-0xFF]
    { c -= 96; }
    else //   (       )
    { c = 95;  }

    for (line=0; line<5; line++)
      LCD_write_data(pgm_read_byte(&font5x7[c][line]));
	CurX +=6;  
  }

void LCD_write_2char(unsigned char c)
  {
    unsigned char line,i,b;

    if ( (c >= 0x20) && (c <= 0x7F) )  //      ASCII[0x20-0x7F]
    {c -= 32;}
    else if ( c >= 0xC0 )   //      CP1251[0xC0-0xFF]
    { c -= 96; }
    else //   (       )
    { c = 95;  }

    for (line=0; line<5; line++) { 
	  i=pgm_read_byte(&font5x7[c][line]);
	  b =  (i & 0x01) * 0b00011;
      b |= (i & 0x02) * 0b00110;
      b |= (i & 0x04) * 0b01100;
      b |= (i & 0x08) * 0b11000;
      LCD_write_data(b);
      LCD_write_data(b);
	}  
	LCD_set_XY(CurX,CurY+1);
    for (line=0; line<5; line++) { 
	  i=pgm_read_byte(&font5x7[c][line]);
	  i >>= 4;
	  b =  (i & 0x01) * 0b00011;
      b |= (i & 0x02) * 0b00110;
      b |= (i & 0x04) * 0b01100;
      b |= (i & 0x08) * 0b11000;
      LCD_write_data(b);
      LCD_write_data(b);
	}  
	LCD_set_XY(CurX+10,CurY-1);
  }

void LCD_write_3char(unsigned char c)
  {
    unsigned char line, col,cols;

    switch (c) {
	  case 'A': c=58-45;
	            break; 
	  case 'V': c=59-45;
	            break; 
	  case 'h': c=60-45;
	            break; 
	  case ' ': c=255;
	            break;
      default:
                if ((c>='-') && (c<='9')) c -='-'; else c=255; 
	}
    if (c==255) cols=14; else cols=pgm_read_byte(font14x24+c*(14*3+1));
    for (line=0; line<3; line++) { 
      for (col=0; col<cols; col++) {
        if (c!=255) LCD_write_data(pgm_read_byte(font14x24+c*(14*3+1)+col*3+line+1)); 
		else LCD_write_data(0);
	  }
	  LCD_set_XY(CurX,CurY+1);
    } 
	LCD_set_XY(CurX+cols,CurY-3);
  }


/*-----------------------------------------------------------------------
LCD_write_english_String  : Display English strings

input parameterX       083
                Y        05

created date         2013-4-25	
-----------------------------------------------------------------------*/
void LCD_write_string(unsigned char X,unsigned char Y,char *s)
  {
    LCD_set_XY(X,Y);
    while (*s) 
      {
	 LCD_write_char(*s);
	 s++;
      }
  }

void LCD_write_3string(unsigned char X,unsigned char Y,char *s)
  {
    LCD_set_XY(X,Y);
    while (*s) 
      {
	 LCD_write_3char(*s);
	 s++;
      }
  }

void LCD_write_2string(unsigned char X,unsigned char Y,char *s)
  {
    LCD_set_XY(X,Y);
    while (*s) 
      {
	 LCD_write_2char(*s);
	 s++;
      }
  }

/*-----------------------------------------------------------------------
LCD_write_chinese_string: Display Chinese strings

	LCD_write_chi(0,0,12,7,0,0);
	LCD_write_chi(0,2,12,7,0,0);
	LCD_write_chi(0,4,12,7,0,0);	
-----------------------------------------------------------------------*/                        
/*
void LCD_write_chinese_string(unsigned char X, unsigned char Y, 
                   unsigned char ch_with,unsigned char num,
                   unsigned char line,unsigned char row)
  {
    unsigned char i,n;
    
    LCD_set_XY(X,Y);                             //scale settings
    
    for (i=0;i<num;)
      {
      	for (n=0; n<ch_with*2; n++)              //write a chinese
      	  { 
      	    if (n==ch_with)                     
      	      {
      	        if (i==0) LCD_set_XY(X,Y+1);
      	        else
      	           LCD_set_XY((X+(ch_with+row)*i),Y+1);
              }
      	    LCD_write_byte(write_chinese[line+i][n],1);
      	  }
      	i++;
      	LCD_set_XY((X+(ch_with+row)*i),Y);
      }
  }
  

*/
/*-----------------------------------------------------------------------
LCD_draw_map      : Bitmap drawing function

input parmenterXY    starting point X,Y
                 *map    bitmap data
                 Pix_x   highth
                 Pix_y   width

created date         2013-4-25
-----------------------------------------------------------------------*/
void LCD_draw_bmp_pixel(unsigned char X,unsigned char Y,unsigned const char *map,
                  unsigned char Pix_x,unsigned char Pix_y)
  {
    unsigned int i,n;
    unsigned char row;
    
    if (Pix_y%8==0) row=Pix_y/8;      //calculate how many line is needed
      else
        row=Pix_y/8+1;
    
    for (n=0;n<row;n++)
      {
      	LCD_set_XY(X,Y);
        for(i=0; i<Pix_x; i++)
          {
            if (map!=NULL) LCD_write_data(pgm_read_byte(&map[i+n*Pix_x])); else LCD_write_data(0);
          }
        Y++;                         //chang line
      }      
  }

