#include <string.h>
#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/eeprom.h>

#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include <avr/sleep.h>
#include <string.h>
#include <nokia_5110.h>
#include <rotimage.h>
#include <batteryimage.h>
#include <brightnessimage.h>
#include <exitimage.h>
 
#define SETBIT(ADDRESS,BIT) (ADDRESS |=(1 << BIT))
#define CLEARBIT(ADDRESS,BIT) (ADDRESS &=~(1 << BIT))
#define TESTBIT(ADDRESS,BIT) (ADDRESS & (1 << BIT))
#define NullOffset 0
#define CurrentLimit 350
#define AvgCount 50
#define RegulatorADC 7
#define CurrentADC 0
#define VoltageADC 1
#define MenuItems 3
#define KeyTimerDelay (F_CPU/128/1300) //(~10 ms)

#define CapacityDivider ((((((F_CPU*36)/KeyTimerDelay)*10)/130)*10)/128)
volatile unsigned char RemuxFlag,RegulatorFlag,CurrentReady,KeyDown,prevD,KeyTimer;
volatile int CurrentS,VoltageS,RegValue;
volatile uint32_t AvgCurrent,AvgVoltage,Timer;

uint16_t dummy EEMEM=0xffff;
uint16_t eeCurrentMult EEMEM=654; //665;
uint16_t eeVoltageMult EEMEM=1554; //3170;
uint16_t eeCapacity EEMEM=0;
uint8_t eeBrightness EEMEM=0;

ISR (ADC_vect) {
 
 if ((RegulatorFlag & 1) == 0) {
	if ((RegulatorFlag & 2)!=0) {
      RegValue=ADC>>2;
	} else {
      VoltageS=ADC;
      AvgVoltage=(AvgVoltage*(AvgCount-1))/AvgCount+VoltageS;	 
    }
    AvgCurrent=(AvgCurrent*(AvgCount-1))/AvgCount+CurrentS;	 
  	CurrentReady=1;
	if ((RegulatorFlag & 2)==0) 
      ADMUX =(1<<REFS0)|RegulatorADC;
	else
      ADMUX =(1<<REFS0)|(1<<REFS0)|VoltageADC;
  } else {
    CurrentS=ADC;
    ADMUX =(1<<REFS0)|CurrentADC;
  } 	
  if (++RegulatorFlag>3) RegulatorFlag=0;
   
  KeyTimer++; 
  if (KeyTimer>=KeyTimerDelay) { 
    if ((prevD&(1<<4)) && !(PIND&(1<<4))) KeyDown=1;
	prevD=PIND;
	KeyTimer=0;
    Timer++;
  } 
} 
 
 
ISR (WDT_vect) {
  wdt_reset();
}  


void putint(int i,uint8_t len, uint8_t dot) {
  int div_val;
  div_val=1;
  dot=len-dot+1;
  len--;
  while (len>0) {div_val *= 10; len--;}
  if (i < 0) {
    i = - i;
    LCD_write_3char('-');
  }
  while (div_val > 1 && div_val > i) {
    if (dot==1) LCD_write_3char('.');  
    dot--;
    div_val /= 10;
	if (dot>1) {
      LCD_write_3char(' ');
	} else 	LCD_write_3char('0');
  }
  do {
    if (dot==1) LCD_write_3char('.');  
    dot--;
    LCD_write_3char(i / div_val+'0');
    i %= div_val;
    div_val /= 10;
  }
  while (div_val);
}

void WaitForNullRegulator(void) {
  LCD_clear();
  if (RegValue) LCD_draw_bmp_pixel(23,1,image_rot,38,32);
  while (RegValue) {
      sleep_cpu();
  	  wdt_reset();
  }
  KeyDown=0;
  wdt_reset();
  LCD_clear();
}  

void SetBrightness(unsigned char b) {
  if (b==0) {
	TCCR0A &=~(1<<COM0A1);
  } else {
	TCCR0A |=(1<<COM0A1);
  }
  OCR0A=b;
}	

void DummyCycle (void) {
  AvgCurrent=0;
  Timer=0;
  while (Timer<20) {
      sleep_cpu();
  	  wdt_reset();
	  LCD_draw_bmp_pixel(0,0,NULL,10,8);
  }
}  

void LCD_write_gap(uint8_t n) {
  LCD_draw_bmp_pixel(CurX,CurY,NULL,n,24);
  LCD_set_XY(CurX+n,CurY-2);
}  
  

int main(void) {
  
  uint16_t i;
  int Current, Voltage=0, CurrentNullOffset, CurrentMult, VoltageMult,Capacity;
  uint32_t CapacityS=0; 
  unsigned char IndicateAh=0, Overload=0, DisplayTimer=0,MenuItem,NewMenuItem,Brightness;
  
  wdt_disable();
  wdt_enable(WDTO_30MS);
  DDRD= 0b11101111;
  PORTD=0b00010000;
  DDRB= 0b00011111;
  PORTB=0b00100000;
  
  TCCR0A=(0<<COM0A1)|(0<<COM0B1)|(1<<WGM01)|(1<<WGM00);
  TCCR0B=(0<<WGM02)|(1<<CS01);
  //SETBIT(TIMSK0,TOIE0);
  ADCSRA |= (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0) | (1 << ADEN) | (1 << ADSC) | (1 << ADIE) | (1<<ADATE);
  ADMUX=(1<<REFS0)|CurrentADC;
  
  CurrentMult=eeprom_read_word(&eeCurrentMult);
  VoltageMult=eeprom_read_word(&eeVoltageMult);
  Capacity=eeprom_read_word(&eeCapacity);
  Brightness=eeprom_read_byte(&eeBrightness);
  SetBrightness(Brightness);
  
  LCD_init();       //LCD initialization
  LCD_clear();      //clear the LCD

  
  sei();
  sleep_enable();
  set_sleep_mode(SLEEP_MODE_IDLE);
  Timer=0;
  
  WaitForNullRegulator();
  DummyCycle();
  CurrentNullOffset=AvgCurrent/AvgCount;

  Timer=0;
  if (!TESTBIT(PINB,5)) {
  // Calibration
	KeyDown=0;
    TCCR0A |=(1<<COM0B1);
	OCR0B=255;
    while (!KeyDown){
      DummyCycle(); 
	  LCD_set_XY(0,0);
	  CurrentMult=618+RegValue/4;
	  Current=((int32_t)(CurrentS-CurrentNullOffset)*CurrentMult)/1024;
	  putint(Current,4,1);
	  LCD_write_3char('A');
	  LCD_set_XY(0,3);
	  putint(CurrentMult,5,0);
	}
	TCCR0A &=~(1<<COM0A1);
	OCR0B=0;
	KeyDown=0;
	eeprom_write_word(&eeCurrentMult,CurrentMult);
    while (!KeyDown){
      DummyCycle();
	  LCD_set_XY(0,0);
	  VoltageMult=292+RegValue/8;
      Voltage=(((int32_t)AvgVoltage*VoltageMult)/AvgCount/1024+5)/10;
	  putint(Voltage,4,1);
	  LCD_write_3char('V');
	  LCD_set_XY(0,3);
	  putint(VoltageMult,5,0);
	}
	wdt_reset();
	eeprom_write_word(&eeVoltageMult,VoltageMult);
	wdt_reset();
    WaitForNullRegulator();
  }  
  KeyDown=0;
  while(1) {
    sleep_cpu();
	wdt_reset();
    if (CurrentReady) {
//	  if (CurrentNullOffset>AvgCurrent/AvgCount) CurrentNullOffset=AvgCurrent/AvgCount;
	  Current=((int32_t)(CurrentS-CurrentNullOffset)*CurrentMult)/1024;
      Voltage=(((int32_t)AvgVoltage*VoltageMult)/AvgCount/1024+5)/10;
	  if (Current<0) Current=0;
	  if (Current<CurrentLimit) {
	    if (RegValue==0) {
		  TCCR0A &=~(1<<COM0B1);
		} else {
		  TCCR0A |=(1<<COM0B1);
		  i=((uint16_t)RegValue*(255-NullOffset))/255+NullOffset;
	      if (OCR0B<i) OCR0B++; else OCR0B=i;
		} 
        LCD_set_XY(0,3);
      } else {
	      i=(Current-CurrentLimit)/2;
	      if (i<OCR0B) OCR0B=OCR0B-i; else OCR0B=0;
          Overload=1;
	  }
 
	  if (Voltage<100) {
		TCCR0A &=~((1<<COM0A1) || (1<<COM0B1));
	    LCD_clear();
		if (eeprom_read_word(&eeCapacity)!=Capacity) eeprom_write_word(&eeCapacity,Capacity);
	  	LCD_set_XY(7,0);
	    putint(Voltage,3,1);
	    LCD_write_3char('V');
		LCD_draw_bmp_pixel(26,3,image_battery,33,24);
		while(1) wdt_reset(); 
	  }	
      CurrentReady=0;
	  if (Timer) {
// 	    if (CurrentNullOffset>AvgCurrent/AvgCount) CurrentNullOffset=AvgCurrent/AvgCount;
        Current=((AvgCurrent/AvgCount-CurrentNullOffset)*CurrentMult)/1024;
        if (Current<0) Current=0; 
        CapacityS +=Current*Timer;
		Timer=0;
        if (CapacityS>CapacityDivider) {
		  Capacity +=CapacityS/CapacityDivider;
		  CapacityS %=CapacityDivider;
		  if (Capacity>=20000) Capacity=0;
		}
        if (KeyDown) {
          if (RegValue==0) {
		    LCD_clear();
		    // Menu
			LCD_draw_bmp_pixel(23,1,image_exit,38,32);

			KeyDown=0;
			MenuItem=0;
			while ((RegValue>0) | !KeyDown) {
			  if (RegValue<((uint16_t)MenuItem*255)/MenuItems ||
              RegValue>(((uint16_t)MenuItem+1)*255)/MenuItems) {
			    NewMenuItem=((uint16_t)RegValue*MenuItems)/255;
			  } else NewMenuItem=MenuItem;
			  if (RegValue!=0 && NewMenuItem==0) NewMenuItem=1;

              if (MenuItem!=NewMenuItem) {
			    MenuItem=NewMenuItem;
                LCD_clear();
				switch (MenuItem) {
				  
				  case 0: LCD_draw_bmp_pixel(23,1,image_exit,38,32);
				          break;
                  case 1: LCD_set_XY(16,2);
				          LCD_write_3char('0');
						  LCD_set_XY(34,2);
						  LCD_write_3char('/');
						  LCD_write_3string(44,2,"Ah");
						  break;
				  case 2: LCD_draw_bmp_pixel(23,1,image_brightness,38,32);
				          break;
		        }
              }
			  if (KeyDown) {
			    if (MenuItem==1) {
				  Capacity=0;
				  WaitForNullRegulator();
				} else if (MenuItem==2) {
                  KeyDown=0;
                  while (!KeyDown) {  
				    Brightness=RegValue;
				    SetBrightness(Brightness);
					wdt_reset();
				  }
					
				  eeprom_write_byte(&eeBrightness,Brightness);
				  WaitForNullRegulator();
				}
              }     
              wdt_reset();	
			}
			  DummyCycle();
              CurrentNullOffset=AvgCurrent/AvgCount;
          } else IndicateAh=!IndicateAh;
		  KeyDown=0;
        }  
	  	LCD_set_XY(0,0);
		
        if (!DisplayTimer) {
	      putint(Voltage,4,1);
		  LCD_write_gap(2);
	      LCD_write_3char('V');
	  	  LCD_set_XY(0,0);
		  if (Overload) LCD_write_3char('/'); else LCD_write_3char(' ');
          Overload=0; 
		  if (IndicateAh) {
	  	    LCD_set_XY(10,3);
		  
	        putint((Capacity+5)/10,3,1);
		    LCD_write_gap(2);
	        LCD_write_3char('A');
	        LCD_write_3char('h');
		  } else {
	  	    LCD_set_XY(0,3);
	        putint(Current,4,1);
		    LCD_write_gap(2);
	        LCD_write_3char('A');
	        LCD_write_3char(' ');
		  }
          DisplayTimer=30;
        } else DisplayTimer--;  
	  }	
    } 
  }

return(0);
}
